// SymbolResolutionDlg.h : header file
//

#pragma once

class CSymbolResolutionDlg;
/**
\typedef ATL::IDispEventImpl<1, CSymbolResolutionDlg, 
                              &__uuidof(_ICQGCELEvents), 
                              &__uuidof(__CQG), 4, 0> ICQGCELDispEventImpl
\brief  Convenient definition of IDispEventImpl instantiation
*/
typedef ATL::IDispEventImpl<1,
                              CSymbolResolutionDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0
                           >
                           ICQGCELDispEventImpl;

/// \brief CSymbolResolutionDlg dialog
class CSymbolResolutionDlg : public CDialog, public ICQGCELDispEventImpl
{
public:

   CSymbolResolutionDlg(
                     CWnd* pParent = NULL);

   /// \brief Dialog Data
   enum { IDD = IDD_SYMBOLRESOLUTION_DIALOG };

   afx_msg void OnBnClickedBtnGetCommodities();
   afx_msg void OnBnClickedBtnGetResolution();
   afx_msg void OnTvnSelChangedTrFullNames(NMHDR *pNMHDR, LRESULT *pResult);
   afx_msg void OnEnChangeEdSymbol();
   afx_msg void OnTvnSelChangedTrCommodities(NMHDR *pNMHDR, LRESULT *pResult);
   afx_msg void OnBnClickedBtnRemove();
   afx_msg void OnBnClickedBtnRemoveAll();
   afx_msg void OnLvnItemchangedLsvaccounts(NMHDR *pNMHDR, LRESULT *pResult);
   afx_msg void OnBnClickedBtnSelectAllAccs();
   afx_msg void OnBnClickedBtnUnselectAllAccs();
   afx_msg void OnPaint();
   afx_msg void OnDestroy();
   afx_msg HCURSOR OnQueryDragIcon();
   DECLARE_MESSAGE_MAP()

protected:

   /// \brief Icon on the form
   HICON m_hIcon;

   /// \brief Generated message map functions
   virtual void DoDataExchange(
                     CDataExchange* pDX);   // DDX/DDV support

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the application has set the input 
   /// \return focus to one of the controls in the dialog box
   virtual BOOL OnInitDialog();

   /// \brief Override to prevent dialog closing on Enter
   virtual void OnOK();

private:

   /// \brief Accounts tree
   CListCtrl m_lvAccounts;
   /// \brief Commodities tree
   CTreeCtrl m_trCommodities;
   /// \brief full name tree
   CTreeCtrl m_trFullNames;
   /// \brief Symbol edit box
   CEdit m_edSymbol;
   /// \brief Get commodities button
   CButton m_btnGetCommodities;
   /// \brief Get instruments button
   CButton m_btnGetInstruments;
   /// \brief Remove button
   CButton m_btnRemove;
   /// \brief Remove all button
   CButton m_btnRemoveAll;
   /// \brief Data connection label
   CStatic m_lblDataConnection;
   /// \brief GW connection label
   CStatic m_lblGWConnection;

private:

   /// \brief Initialize all controls
   void InitializeControls();

   /// \brief Initialize CQGCEL object, and starts the CQGCEL
   void InitializeCQGCEL();

   /// \brief Checks if the connection with CQG data line is up
   /// \return Returns true if connection is up
   BOOL IsDataConnectionUp() const;

   /// \brief Checks if the connection with CQG Gateway server is up
   /// \return Returns true if connection is up
   BOOL IsGWConnectionUp() const;

   /// \brief Counts the checked accounts in the accounts list
   /// \return Checked accounts count
   int GetCheckedAccountsCount() const;

   /// \brief Adds account to the accounts list
   /// \param spAccount - Account to add
   void InsertAccount(
                    const ATL::CComPtr<ICQGAccount>& spAccount);

   /// \brief Checks/Unchecks all accounts
   /// \param check - Check status
   void CheckAllAccounts(
                    BOOL check);

   /// \brief Returns string representation of the specified instrument type
   /// \param type - instrument type
   /// \return Instrument type string
   static CString InstrumentTypeToString(
                    eInstrumentType type);

   /// \brief Returns string representation of the connection status.    
   /// \param status - connection status
   /// \return String representation of the connection status
   static CString ConnectionStatusToString(
                    eConnectionStatus status);

public:

   /// \brief This map is used to declare handler function for event specified event.
   /// \brief You can use "OLE/COM Object Viewer" to open CQGCEL-4_0.dll and get event ids.
   /// \brief The handler signature as well as the event ids can be found in the type library
   /// \brief or shown by "OLE/COM Object Viewer".
   BEGIN_SINK_MAP(CSymbolResolutionDlg)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 10, OnDataError)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 2, OnGWConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 3, OnDataConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 7, OnAccountChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 13, OnCommodityInstrumentsResolved)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 21, OnTradableCommoditiesResolved)
   END_SINK_MAP()

protected:

   /// \brief Handle event of some abnormal discrepancy
   /// \brief between data expected and data received from CQG.
   /// \brief This event also can be fired when CQGCEL startup fails.
   /// \param obj - Object where the error occurred
   /// \param errorDescription - String describing the error
   /// \return S_OK
   STDMETHOD(OnDataError)(
                        LPDISPATCH obj,
                        BSTR errorDescription);

   /// \brief Fired when some changes occurred
   /// \brief in the connection with the CQG Gateway is changed.
   /// \param newStatus - New connection status
   /// \return S_OK
   STDMETHOD(OnGWConnectionStatusChanged)(
                        eConnectionStatus newStatus);

   /// \brief Fired when some changes occurred
   /// \brief in the connection with the CQG data server.
   /// \param newStatus - New connection status
   /// \return S_OK
   STDMETHOD(OnDataConnectionStatusChanged)(
                        eConnectionStatus newStatus);

   /// \brief Fired when instruments list for requested commodity is resolved.
   /// \param commodityName - Requested symbol
   /// \param instrumentTypes - Requested instruments types
   /// \param instrumentNames - Collection of resolved instruments
   /// \return S_OK
   STDMETHOD(OnCommodityInstrumentsResolved)(
                        BSTR commodityName,
                        eInstrumentType instrumentTypes,
                        ICQGCommodityInstruments* instrumentNames);

   /// \brief Fired when new account is added, changed or removed.
   /// \param change - Account change type
   /// \param Account - Changed account instance
   /// \param position - Changed position instance
   /// \return S_OK
   STDMETHOD(OnAccountChanged)(
                        eAccountChangeType change,
                        ICQGAccount* account,
                        ICQGPosition* position);

   /// \brief Fired when list of tradable commodities is resolved.
   /// \param GWAccountID - Account Gateway ID which commodities were requested
   /// \param commodityNames - Collection of resolved commodities
   /// \param err - CQGError instance representing the error occurred
   /// \param while processing the time series request
   /// \return S_OK
   STDMETHOD(OnTradableCommoditiesResolved)(
                        long gwAccountID,
                        ICQGCommodities* commodityNames,
                        ICQGError* err);

private:

   /// \brief CQGCEL object
   ATL::CComPtr<ICQGCEL> m_spCQGCEL;

};
